@@ -21,9 +21,10 @@
 #include <QStringList>
 #include <QColor>
 
+#include <memory>
 #include "qgis_sip.h"
 #include "qgsconfig.h"
-#include "qgssettingsentry.h"
+#include "qgssettingsentryimpl.h"
 #include "qgstranslationcontext.h"
 
 class QgsSettingsRegistryCore;
@@ -36,7 +37,9 @@ class QgsFieldFormatterRegistry;
 class QgsColorSchemeRegistry;
 class QgsPaintEffectRegistry;
 class QgsProjectStorageRegistry;
+class QgsExternalStorageRegistry;
 class QgsLocalizedDataPathRegistry;
+class QgsLayerMetadataProviderRegistry;
 class QgsRendererRegistry;
 class QgsSvgCache;
 class QgsImageCache;
@@ -44,6 +47,7 @@ class QgsSourceCache;
 class QgsSymbolLayerRegistry;
 class QgsRasterRendererRegistry;
 class QgsGpsConnectionRegistry;
+class QgsBabelFormatRegistry;
 class QgsDataItemProviderRegistry;
 class QgsPluginLayerRegistry;
 class QgsClassificationMethodRegistry;
@@ -66,8 +70,14 @@ class QgsConnectionRegistry;
 class QgsScaleBarRendererRegistry;
 class Qgs3DSymbolRegistry;
 class QgsPointCloudRendererRegistry;
+class QgsTiledSceneRendererRegistry;
 class QgsTileDownloadManager;
 class QgsCoordinateReferenceSystemRegistry;
+class QgsRecentStyleHandler;
+class QgsDatabaseQueryLog;
+class QgsFontManager;
+class QgsSensorRegistry;
+class QgsProfileSourceRegistry;
 
 /**
  * \ingroup core
@@ -156,9 +166,28 @@ class CORE_EXPORT QgsApplication : public QApplication
     static const char *QGIS_ORGANIZATION_DOMAIN;
     static const char *QGIS_APPLICATION_NAME;
 #ifndef SIP_RUN
-    QgsApplication( int &argc, char **argv, bool GUIenabled, const QString &profileFolder = QString(), const QString &platformName = "desktop" );
+
+    /**
+     * Constructor for QgsApplication.
+     *
+     * \param argc command line argument count
+     * \param argv command line arguments
+     * \param GUIenabled set to TRUE if a GUI application is required, or FALSE for a console only application
+     * \param profileFolder optional string representing the profile to load at startup
+     * \param platformName the QGIS platform name, e.g., "desktop", "server", "qgis_process" or "external" (for external CLI scripts)
+     */
+    QgsApplication( int &argc, char **argv, bool GUIenabled, const QString &profileFolder = QString(), const QString &platformName = "external" );
 #else
-    QgsApplication( SIP_PYLIST argv, bool GUIenabled, QString profileFolder = QString(), QString platformName = "desktop" ) / PostHook = __pyQtQAppHook__ / [( int &argc, char **argv, bool GUIenabled, const QString &profileFolder = QString(), const QString &platformName = "desktop" )];
+
+    /**
+     * Constructor for QgsApplication.
+     *
+     * \param argv command line arguments
+     * \param GUIenabled set to TRUE if a GUI application is required, or FALSE for a console only application
+     * \param profileFolder optional string representing the profile to load at startup
+     * \param platformName the QGIS platform name, e.g., "desktop", "server", "qgis_process" or "external" (for external CLI scripts)
+     */
+    QgsApplication( SIP_PYLIST argv, bool GUIenabled, QString profileFolder = QString(), QString platformName = "external" ) / PostHook = __pyQtQAppHook__ / [( int &argc, char **argv, bool GUIenabled, const QString &profileFolder = QString(), const QString &platformName = "desktop" )];
     % MethodCode
     // The Python interface is a list of argument strings that is modified.
 
@@ -186,7 +215,6 @@ class CORE_EXPORT QgsApplication : public QApplication
     /**
      * Returns the singleton instance of the QgsApplication.
      *
-     * \since QGIS 3.0
      */
     static QgsApplication *instance();
 
@@ -266,7 +294,6 @@ class CORE_EXPORT QgsApplication : public QApplication
      * Returns the path to the developers map file.
      * The developers map was created by using leaflet framework,
      * it shows the contributors.json file.
-     * \since QGIS 2.7
     */
     static QString developersMapFilePath();
 
@@ -294,7 +321,6 @@ class CORE_EXPORT QgsApplication : public QApplication
 
     /**
      * Returns the path to the metadata directory.
-    * \since QGIS 3.0
     */
     static QString metadataPath();
 
@@ -331,7 +357,6 @@ class CORE_EXPORT QgsApplication : public QApplication
 
     /**
      * Returns the paths to layout template directories.
-     * \since QGIS 3.0
      */
     static QStringList layoutTemplatePaths();
 
@@ -404,43 +429,73 @@ class CORE_EXPORT QgsApplication : public QApplication
     //! Returns the path to user's style.
     static QString userStylePath();
 
-    //! Returns the short name regular expression for line edit validator
-    static QRegExp shortNameRegExp();
+    /**
+     * Returns the short name regular expression for line edit validator
+     * \note This functionality was previously available as `shortNameRegExp` for QGIS <= 3.20
+     * \since QGIS 3.22
+     */
+    static QRegularExpression shortNameRegularExpression();
 
     /**
      * Returns the user's operating system login account name.
      * \see userFullName()
-     * \since QGIS 2.14
      */
     static QString userLoginName();
 
     /**
      * Returns the user's operating system login account full display name.
      * \see userLoginName()
-     * \since QGIS 2.14
      */
     static QString userFullName();
 
     /**
      * Returns a string name of the operating system QGIS is running on.
      * \see platform()
-     * \since QGIS 2.14
      */
     static QString osName();
 
     /**
-     * Returns the QGIS platform name, e.g., "desktop" or "server".
+     * Returns the size of the system memory (RAM) in megabytes.
+     *
+     * This is only supported on some platforms, and will return -1 if not supported.
+     *
+     * \since QGIS 3.26
+     */
+    static int systemMemorySizeMb();
+
+    /**
+     * Returns the QGIS platform name, e.g., "desktop", "server", "qgis_process" or "external" (for external CLI scripts).
      * \see osName()
-     * \since QGIS 2.14
      */
     static QString platform();
 
+
+    /**
+     * Returns the QGIS application full name.
+     *
+     * It can be defined by the environment variable QGIS_APPLICATION_FULL_NAME or the /qgis/application_full_name
+     * in the QGIS config file.
+     *
+     * By default it is equal to applicationName()+' '+platform()
+     *
+     * \see platform()
+     * \since QGIS 3.30
+     */
+    static QString applicationFullName();
+
     /**
      * Returns the QGIS locale.
-     * \since QGIS 3.0
      */
     static QString locale();
 
+    /**
+     * Sets the QGIS locale - used mainly by 3rd party apps and tests.
+     * In QGIS this is internally triggered by the application in startup.
+     *
+     * \since QGIS 3.22.2
+     */
+    static void setLocale( const QLocale &locale );
+
     //! Returns the path to user's themes folder
     static QString userThemesFolder();
 
@@ -603,52 +658,46 @@ class CORE_EXPORT QgsApplication : public QApplication
 
     /**
      * Gets maximum concurrent thread count
-     * \since QGIS 2.4
     */
     static int maxThreads();
 
     /**
      * Set maximum concurrent thread count
-     * \note must be between 1 and \#cores, -1 means use all available cores
-     * \since QGIS 2.4
+     * \note must be between 2 and \#cores, -1 means use all available cores
     */
     static void setMaxThreads( int maxThreads );
 
     /**
      * Returns the application's task manager, used for managing application
      * wide background task handling.
-     * \since QGIS 3.0
      */
     static QgsTaskManager *taskManager();
 
     /**
      * Returns the application's settings registry, used for managing application settings.
      * \since QGIS 3.20
+     * \deprecated since QGIS 3.30 use QgsSettings::treeRoot() instead
      */
-    static QgsSettingsRegistryCore *settingsRegistryCore() SIP_KEEPREFERENCE;
+    Q_DECL_DEPRECATED static QgsSettingsRegistryCore *settingsRegistryCore() SIP_KEEPREFERENCE SIP_DEPRECATED;
 
     /**
      * Returns the application's color scheme registry, used for managing color schemes.
-     * \since QGIS 3.0
      */
     static QgsColorSchemeRegistry *colorSchemeRegistry() SIP_KEEPREFERENCE;
 
     /**
      * Returns the application's paint effect registry, used for managing paint effects.
-     * \since QGIS 3.0
      */
     static QgsPaintEffectRegistry *paintEffectRegistry() SIP_KEEPREFERENCE;
 
     /**
      * Returns the application's renderer registry, used for managing vector layer renderers.
-     * \since QGIS 3.0
      */
     static QgsRendererRegistry *rendererRegistry() SIP_KEEPREFERENCE;
 
     /**
      * Returns the application's raster renderer registry, used for managing raster layer renderers.
      * \note not available in Python bindings
-     * \since QGIS 3.0
      */
     static QgsRasterRendererRegistry *rasterRendererRegistry() SIP_SKIP;
 
@@ -658,10 +707,15 @@ class CORE_EXPORT QgsApplication : public QApplication
      */
     static QgsPointCloudRendererRegistry *pointCloudRendererRegistry() SIP_KEEPREFERENCE;
 
+    /**
+     * Returns the application's tiled scene renderer registry, used for managing tiled scene layer 2D renderers.
+     * \since QGIS 3.34
+     */
+    static QgsTiledSceneRendererRegistry *tiledSceneRendererRegistry() SIP_KEEPREFERENCE;
+
     /**
      * Returns the application's data item provider registry, which keeps a list of data item
      * providers that may add items to the browser tree.
-     * \since QGIS 3.0
      */
     static QgsDataItemProviderRegistry *dataItemProviderRegistry() SIP_KEEPREFERENCE;
 
@@ -678,7 +732,6 @@ class CORE_EXPORT QgsApplication : public QApplication
      * within SVG files.
      *
      * \see imageCache()
-     * \since QGIS 3.0
      */
     static QgsSvgCache *svgCache();
 
@@ -711,7 +764,6 @@ class CORE_EXPORT QgsApplication : public QApplication
 
     /**
      * Returns the application's symbol layer registry, used for managing symbol layers.
-     * \since QGIS 3.0
      */
     static QgsSymbolLayerRegistry *symbolLayerRegistry() SIP_KEEPREFERENCE;
 
@@ -723,7 +775,6 @@ class CORE_EXPORT QgsApplication : public QApplication
 
     /**
      * Returns the application's layout item registry, used for layout item types.
-     * \since QGIS 3.0
      */
     static QgsLayoutItemRegistry *layoutItemRegistry() SIP_KEEPREFERENCE;
 
@@ -735,13 +786,17 @@ class CORE_EXPORT QgsApplication : public QApplication
 
     /**
      * Returns the application's GPS connection registry, used for managing GPS connections.
-     * \since QGIS 3.0
      */
     static QgsGpsConnectionRegistry *gpsConnectionRegistry() SIP_KEEPREFERENCE;
 
+    /**
+     * Returns the application's GPSBabel format registry, used for managing GPSBabel formats.
+     * \since QGIS 3.22
+     */
+    static QgsBabelFormatRegistry *gpsBabelFormatRegistry() SIP_KEEPREFERENCE;
+
     /**
      * Returns the application's plugin layer registry, used for managing plugin layer types.
-     * \since QGIS 3.0
      */
     static QgsPluginLayerRegistry *pluginLayerRegistry() SIP_KEEPREFERENCE;
 
@@ -764,6 +819,19 @@ class CORE_EXPORT QgsApplication : public QApplication
      */
     static QgsTileDownloadManager *tileDownloadManager() SIP_SKIP;
 
+    /**
+     * Returns the handler for recently used style items.
+     * \since QGIS 3.22
+     */
+    static QgsRecentStyleHandler *recentStyleHandler() SIP_KEEPREFERENCE;
+
+    /**
+     * Returns the database query log.
+     *
+     * \since QGIS 3.24
+     */
+    static QgsDatabaseQueryLog *databaseQueryLog() SIP_KEEPREFERENCE;
+
     /**
      * Returns a shared QgsStyleModel containing the default style library (see QgsStyle::defaultStyle()).
      *
@@ -773,9 +841,21 @@ class CORE_EXPORT QgsApplication : public QApplication
      */
     static QgsStyleModel *defaultStyleModel();
 
+    /**
+     * Returns the application font manager, which manages available fonts and font installation for the QGIS instance.
+     *
+     * \since QGIS 3.28
+     */
+    static QgsFontManager *fontManager() SIP_KEEPREFERENCE;
+
+    /**
+     * Returns the application's sensor registry, used for sensor types.
+     * \since QGIS 3.32
+     */
+    static QgsSensorRegistry *sensorRegistry() SIP_KEEPREFERENCE;
+
     /**
      * Returns the application's message log.
-     * \since QGIS 3.0
      */
     static QgsMessageLog *messageLog();
 
@@ -783,34 +863,29 @@ class CORE_EXPORT QgsApplication : public QApplication
      * Returns the application's authentication manager instance
      * \note this can be NULLPTR if called before initQgis
      * \see initQgis
-     * \since QGIS 3.0
      */
     static QgsAuthManager *authManager();
 
     /**
      * Returns the application's processing registry, used for managing processing providers,
      * algorithms, and various parameters and outputs.
-     * \since QGIS 3.0
      */
     static QgsProcessingRegistry *processingRegistry();
 
     /**
      * Returns the application's page size registry, used for managing layout page sizes.
-     * \since QGIS 3.0
      */
     static QgsPageSizeRegistry *pageSizeRegistry() SIP_KEEPREFERENCE;
 
     /**
      * Returns the application's annotation registry, used for managing annotation types.
      * \note not available in Python bindings
-     * \since QGIS 3.0
      */
     static QgsAnnotationRegistry *annotationRegistry() SIP_SKIP;
 
     /**
      * Returns the action scope registry.
      *
-     * \since QGIS 3.0
      */
     static QgsActionScopeRegistry *actionScopeRegistry() SIP_KEEPREFERENCE;
 
@@ -822,7 +897,6 @@ class CORE_EXPORT QgsApplication : public QApplication
 
     /**
      * Returns the application runtime profiler.
-     * \since QGIS 3.0
      */
     static QgsRuntimeProfiler *profiler();
 
@@ -840,7 +914,6 @@ class CORE_EXPORT QgsApplication : public QApplication
 
     /**
      * Returns registry of available 3D renderers.
-     * \since QGIS 3.0
      */
     static Qgs3DRendererRegistry *renderer3DRegistry() SIP_KEEPREFERENCE;
 
@@ -863,6 +936,24 @@ class CORE_EXPORT QgsApplication : public QApplication
      */
     static QgsProjectStorageRegistry *projectStorageRegistry() SIP_KEEPREFERENCE;
 
+    /**
+     * Returns registry of available layer metadata provider implementations.
+     * \since QGIS 3.28
+     */
+    static QgsLayerMetadataProviderRegistry *layerMetadataProviderRegistry() SIP_KEEPREFERENCE;
+
+    /**
+     * Returns registry of available external storage implementations.
+     * \since QGIS 3.20
+     */
+    static QgsExternalStorageRegistry *externalStorageRegistry() SIP_KEEPREFERENCE;
+
+    /**
+     * Returns registry of available profile source implementations.
+     * \since QGIS 3.38
+     */
+    static QgsProfileSourceRegistry *profileSourceRegistry() SIP_KEEPREFERENCE;
+
     /**
      * Returns the registry of data repositories
      * These are used as paths for basemaps, logos, etc. which can be referenced
@@ -892,7 +983,6 @@ class CORE_EXPORT QgsApplication : public QApplication
      * This does not include generated variables (like system name, user name etc.)
      *
      * \see QgsExpressionContextUtils::globalScope().
-     * \since QGIS 3.0
      */
     static QVariantMap customVariables();
 
@@ -901,14 +991,12 @@ class CORE_EXPORT QgsApplication : public QApplication
      * Do not include generated variables (like system name, user name etc.)
      *
      * \see QgsExpressionContextUtils::globalScope().
-     * \since QGIS 3.0
      */
     static void setCustomVariables( const QVariantMap &customVariables );
 
     /**
      * Set a single custom expression variable.
      *
-     * \since QGIS 3.0
      */
     static void setCustomVariable( const QString &name, const QVariant &value );
 
@@ -934,12 +1022,19 @@ class CORE_EXPORT QgsApplication : public QApplication
     int maxConcurrentConnectionsPerPool() const;
 
     /**
-     * Set translation
+     * Set translation locale code
      *
      * \since QGIS 3.4
      */
     static void setTranslation( const QString &translation );
 
+    /**
+     * Returns the current application translation locale code
+     * \see setTranslation()
+     * \since QGIS 3.22
+     */
+    QString translation() const;
+
     /**
      * Emits the signal to collect all the strings of .qgs to be included in ts file
      *
@@ -947,17 +1042,18 @@ class CORE_EXPORT QgsApplication : public QApplication
      */
     void collectTranslatableObjects( QgsTranslationContext *translationContext );
 
+
 #ifndef SIP_RUN
     //! Settings entry locale user locale
-    static const inline QgsSettingsEntryString settingsLocaleUserLocale = QgsSettingsEntryString( QStringLiteral( "locale/userLocale" ), QgsSettings::NoSection, QString() );
+    static const QgsSettingsEntryString *settingsLocaleUserLocale;
     //! Settings entry locale override flag
-    static const inline QgsSettingsEntryBool settingsLocaleOverrideFlag = QgsSettingsEntryBool( QStringLiteral( "locale/overrideFlag" ), QgsSettings::NoSection, false );
+    static const QgsSettingsEntryBool *settingsLocaleOverrideFlag;
     //! Settings entry locale global locale
-    static const inline QgsSettingsEntryString settingsLocaleGlobalLocale = QgsSettingsEntryString( QStringLiteral( "locale/globalLocale" ), QgsSettings::NoSection, QString() );
+    static const QgsSettingsEntryString *settingsLocaleGlobalLocale;
     //! Settings entry locale show group separator
-    static const inline QgsSettingsEntryBool settingsLocaleShowGroupSeparator = QgsSettingsEntryBool( QStringLiteral( "locale/showGroupSeparator" ), QgsSettings::NoSection, false );
+    static const QgsSettingsEntryBool *settingsLocaleShowGroupSeparator;
     //! Settings entry search path for SVG
-    static const inline QgsSettingsEntryStringList settingsSearchPathsForSVG = QgsSettingsEntryStringList( QStringLiteral( "svg/searchPathsForSVG" ), QgsSettings::NoSection, QStringList() );
+    static const QgsSettingsEntryStringList *settingsSearchPathsForSVG;
 #endif
 
 #ifdef SIP_RUN
@@ -973,7 +1069,6 @@ class CORE_EXPORT QgsApplication : public QApplication
 
     /**
      * Emitted whenever a custom global variable changes.
-     * \since QGIS 3.0
      */
     void customVariablesChanged();
 
@@ -991,6 +1086,15 @@ class CORE_EXPORT QgsApplication : public QApplication
      */
     void requestForTranslatableObjects( QgsTranslationContext *translationContext );
 
+
+    /**
+     * Emitted when project locale has been changed.
+     *
+     * \since QGIS 3.22.2
+     */
+    void localeChanged();
+
+
   private:
 
     static void copyPath( const QString &src, const QString &dst );
@@ -1002,7 +1106,6 @@ class CORE_EXPORT QgsApplication : public QApplication
     static bool ABISYM( mRunningFromBuildDir );
 
     /**
-     * \since QGIS 2.4
     */
     static int ABISYM( sMaxThreads );
 
@@ -1011,6 +1114,7 @@ class CORE_EXPORT QgsApplication : public QApplication
 
     QTranslator *mQgisTranslator = nullptr;
     QTranslator *mQtTranslator = nullptr;
+    QTranslator *mQtBaseTranslator = nullptr;
 
     QgsDataItemProviderRegistry *mDataItemProviderRegistry = nullptr;
     QgsAuthManager *mAuthManager = nullptr;
@@ -1028,6 +1132,7 @@ class CORE_EXPORT QgsApplication : public QApplication
       QgsNumericFormatRegistry *mNumericFormatRegistry = nullptr;
       QgsFieldFormatterRegistry *mFieldFormatterRegistry = nullptr;
       QgsGpsConnectionRegistry *mGpsConnectionRegistry = nullptr;
+      QgsBabelFormatRegistry *mGpsBabelFormatRegistry = nullptr;
       QgsNetworkContentFetcherRegistry *mNetworkContentFetcherRegistry = nullptr;
       QgsScaleBarRendererRegistry *mScaleBarRendererRegistry = nullptr;
       QgsValidityCheckRegistry *mValidityCheckRegistry = nullptr;
@@ -1038,10 +1143,14 @@ class CORE_EXPORT QgsApplication : public QApplication
       QgsProcessingRegistry *mProcessingRegistry = nullptr;
       QgsConnectionRegistry *mConnectionRegistry = nullptr;
       QgsProjectStorageRegistry *mProjectStorageRegistry = nullptr;
+      QgsLayerMetadataProviderRegistry *mLayerMetadataProviderRegistry = nullptr;
+      QgsExternalStorageRegistry *mExternalStorageRegistry = nullptr;
+      QgsProfileSourceRegistry *mProfileSourceRegistry = nullptr;
       QgsPageSizeRegistry *mPageSizeRegistry = nullptr;
       QgsRasterRendererRegistry *mRasterRendererRegistry = nullptr;
       QgsRendererRegistry *mRendererRegistry = nullptr;
       QgsPointCloudRendererRegistry *mPointCloudRendererRegistry = nullptr;
+      QgsTiledSceneRendererRegistry *mTiledSceneRendererRegistry = nullptr;
       QgsSvgCache *mSvgCache = nullptr;
       QgsImageCache *mImageCache = nullptr;
       QgsSourceCache *mSourceCache = nullptr;
@@ -1050,10 +1159,14 @@ class CORE_EXPORT QgsApplication : public QApplication
       QgsTaskManager *mTaskManager = nullptr;
       QgsLayoutItemRegistry *mLayoutItemRegistry = nullptr;
       QgsAnnotationItemRegistry *mAnnotationItemRegistry = nullptr;
+      QgsSensorRegistry *mSensorRegistry = nullptr;
       QgsUserProfileManager *mUserConfigManager = nullptr;
       QgsBookmarkManager *mBookmarkManager = nullptr;
       QgsTileDownloadManager *mTileDownloadManager = nullptr;
       QgsStyleModel *mStyleModel = nullptr;
+      QgsRecentStyleHandler *mRecentStyleHandler = nullptr;
+      QgsDatabaseQueryLog *mQueryLogger = nullptr;
+      QgsFontManager *mFontManager;
       QString mNullRepresentation;
       QStringList mSvgPathCache;
       bool mSvgPathCacheValid = false;
@@ -1073,6 +1186,12 @@ class CORE_EXPORT QgsApplication : public QApplication
 
     static void invalidateCaches();
 
+    /**
+     * Installs the translators (qgis, qt and qt_base) for the current locale.
+     * \since QGIS 3.22
+    */
+    void installTranslators() SIP_SKIP;
+
     friend class TestQgsApplication;
 };
 
