@@ -25,7 +25,6 @@
 #include "qgis_core.h"
 #include "qgis.h"
 #include "qgscolorrampshader.h"
-#include "qgsmeshdataprovider.h"
 #include "qgsmesh3daveraging.h"
 #include "qgsinterpolatedlinerenderer.h"
 
@@ -61,14 +60,14 @@ class CORE_EXPORT QgsMeshRendererMeshSettings
      *
      * \since QGIS 3.14
      */
-    QgsUnitTypes::RenderUnit lineWidthUnit() const;
+    Qgis::RenderUnit lineWidthUnit() const;
 
     /**
      * Sets units of the width of the mesh frame
      *
      * \since QGIS 3.14
      */
-    void setLineWidthUnit( const QgsUnitTypes::RenderUnit &lineWidthUnit );
+    void setLineWidthUnit( Qgis::RenderUnit lineWidthUnit );
 
     //! Writes configuration to a new DOM element
     QDomElement writeXml( QDomDocument &doc ) const;
@@ -78,7 +77,7 @@ class CORE_EXPORT QgsMeshRendererMeshSettings
   private:
     bool mEnabled = false;
     double mLineWidth = DEFAULT_LINE_WIDTH;
-    QgsUnitTypes::RenderUnit mLineWidthUnit = QgsUnitTypes::RenderMillimeters;
+    Qgis::RenderUnit mLineWidthUnit = Qgis::RenderUnit::Millimeters;
     QColor mColor = Qt::black;
 };
 
@@ -108,7 +107,7 @@ class CORE_EXPORT QgsMeshRendererScalarSettings
       /**
        * Does not use resampling
        */
-      None = 0,
+      NoResampling = 0,
 
       /**
        * Does a simple average of values defined for all surrounding faces/vertices
@@ -168,14 +167,14 @@ class CORE_EXPORT QgsMeshRendererScalarSettings
     *
     * \since QGIS 3.14
     */
-    QgsUnitTypes::RenderUnit edgeStrokeWidthUnit() const;
+    Qgis::RenderUnit edgeStrokeWidthUnit() const;
 
     /**
      * Sets the stroke width unit used to render edges scalar dataset
      *
      * \since QGIS 3.14
      */
-    void setEdgeStrokeWidthUnit( const QgsUnitTypes::RenderUnit &edgeStrokeWidthUnit );
+    void setEdgeStrokeWidthUnit( Qgis::RenderUnit edgeStrokeWidthUnit );
 
     //! Writes configuration to a new DOM element
     QDomElement writeXml( QDomDocument &doc, const QgsReadWriteContext &context = QgsReadWriteContext() ) const;
@@ -184,13 +183,13 @@ class CORE_EXPORT QgsMeshRendererScalarSettings
 
   private:
     QgsColorRampShader mColorRampShader;
-    DataResamplingMethod mDataResamplingMethod = DataResamplingMethod::None;
+    DataResamplingMethod mDataResamplingMethod = DataResamplingMethod::NoResampling;
     double mClassificationMinimum = 0;
     double mClassificationMaximum = 0;
     double mOpacity = 1;
 
     QgsInterpolatedLineWidth mEdgeStrokeWidth;
-    QgsUnitTypes::RenderUnit mEdgeStrokeWidthUnit = QgsUnitTypes::RenderMillimeters;
+    Qgis::RenderUnit mEdgeStrokeWidthUnit = Qgis::RenderUnit::Millimeters;
 };
 
 /**
@@ -380,24 +379,98 @@ class CORE_EXPORT QgsMeshRendererVectorTracesSettings
     //! Sets particles count
     void setParticlesCount( int value );
     //! Returns the maximum tail length unit
-    QgsUnitTypes::RenderUnit maximumTailLengthUnit() const;
+    Qgis::RenderUnit maximumTailLengthUnit() const;
     //! Sets the maximum tail length unit
-    void setMaximumTailLengthUnit( const QgsUnitTypes::RenderUnit &maximumTailLengthUnit );
+    void setMaximumTailLengthUnit( Qgis::RenderUnit maximumTailLengthUnit );
 
     //! Reads configuration from the given DOM element
     void readXml( const QDomElement &elem );
     //! Writes configuration to a new DOM element
     QDomElement writeXml( QDomDocument &doc ) const;
 
-
-
   private:
     int mParticlesCount = 1000;
     double mMaximumTailLength = 100;
-    QgsUnitTypes::RenderUnit mMaximumTailLengthUnit = QgsUnitTypes::RenderMillimeters;
+    Qgis::RenderUnit mMaximumTailLengthUnit = Qgis::RenderUnit::Millimeters;
 
 };
 
+/**
+ * \ingroup core
+ *
+ * \brief Represents a mesh renderer settings for vector datasets displayed with wind barbs
+ *
+ * \note The API is considered EXPERIMENTAL and can be changed without a notice
+ *
+ * \since QGIS 3.38
+ */
+class CORE_EXPORT QgsMeshRendererVectorWindBarbSettings
+{
+  public:
+    //! Wind speed units. Wind barbs use knots so we use this enum for preset conversion values
+    enum class WindSpeedUnit
+    {
+      MetersPerSecond = 0, //!< Meters per second
+      KilometersPerHour, //!< Kilometers per hour
+      Knots, //!< Knots (Nautical miles per hour)
+      MilesPerHour, //!< Miles per hour
+      FeetPerSecond, //!< Feet per second
+      OtherUnit //!< Other unit
+    };
+
+    /**
+     * Returns the multiplier for the magnitude to convert it to knots, according to the units set with setMagnitudeUnits()
+     * A custom multiplier can be set with setMagnitudeMultiplier() for the case when units are set to OtherUnit
+     */
+    double magnitudeMultiplier() const;
+
+    /**
+     * Sets a multiplier for the magnitude to convert it to knots
+     */
+    void setMagnitudeMultiplier( double magnitudeMultiplier );
+
+    /**
+     * Returns the shaft length (in millimeters)
+     */
+    double shaftLength() const;
+
+    /**
+     * Sets the shaft length  (in millimeters)
+     */
+    void setShaftLength( double shaftLength );
+
+    /**
+     * Sets the units for the shaft length
+     */
+    Qgis::RenderUnit shaftLengthUnits();
+
+    /**
+     * Returns the units for the shaft length
+     */
+    void setShaftLengthUnits( Qgis::RenderUnit shaftLengthUnit );
+
+    /**
+     * Returns the units that the data are in
+     */
+    WindSpeedUnit magnitudeUnits() const;
+
+    /**
+     * Sets the units that the data are in
+     */
+    void setMagnitudeUnits( WindSpeedUnit units );
+
+    //! Writes configuration to a new DOM element
+    QDomElement writeXml( QDomDocument &doc ) const;
+    //! Reads configuration from the given DOM element
+    void readXml( const QDomElement &elem );
+
+  private:
+    double mShaftLength = 10;
+    Qgis::RenderUnit mShaftLengthUnits = Qgis::RenderUnit::Millimeters;
+    WindSpeedUnit mMagnitudeUnits = WindSpeedUnit::MetersPerSecond;
+    double mMagnitudeMultiplier = 1;
+};
+
 /**
  * \ingroup core
  *
@@ -422,7 +495,9 @@ class CORE_EXPORT QgsMeshRendererVectorSettings
       //! Displaying vector dataset with streamlines
       Streamlines,
       //! Displaying vector dataset with particle traces
-      Traces
+      Traces,
+      //! Displaying vector dataset with wind barbs
+      WindBarbs
     };
 
     //! Returns line width of the arrow (in millimeters)
@@ -554,6 +629,18 @@ class CORE_EXPORT QgsMeshRendererVectorSettings
      */
     void setTracesSettings( const QgsMeshRendererVectorTracesSettings &tracesSettings );
 
+    /**
+    * Returns settings for vector rendered with wind barbs
+    * \since QGIS 3.38
+    */
+    QgsMeshRendererVectorWindBarbSettings windBarbSettings() const;
+
+    /**
+     * Sets settings for vector rendered with wind barbs
+     * \since QGIS 3.38
+     */
+    void setWindBarbSettings( const QgsMeshRendererVectorWindBarbSettings &windBarbSettings );
+
     //! Writes configuration to a new DOM element
     QDomElement writeXml( QDomDocument &doc, const QgsReadWriteContext &context = QgsReadWriteContext() ) const;
     //! Reads configuration from the given DOM element
@@ -576,6 +663,7 @@ class CORE_EXPORT QgsMeshRendererVectorSettings
     QgsMeshRendererVectorArrowSettings mArrowsSettings;
     QgsMeshRendererVectorStreamlineSettings mStreamLinesSettings;
     QgsMeshRendererVectorTracesSettings mTracesSettings;
+    QgsMeshRendererVectorWindBarbSettings mWindBarbSettings;
 };
 
 /**
@@ -622,27 +710,52 @@ class CORE_EXPORT QgsMeshRendererSettings
 
     //! Returns renderer settings
     QgsMeshRendererScalarSettings scalarSettings( int groupIndex ) const { return mRendererScalarSettings.value( groupIndex ); }
+
     //! Sets new renderer settings
     void setScalarSettings( int groupIndex, const QgsMeshRendererScalarSettings &settings ) { mRendererScalarSettings[groupIndex] = settings; }
 
+    /**
+     * Returns whether \a groupIndex has existing scalar settings
+     * \since QGIS 3.30.2
+     */
+    bool hasScalarSettings( int groupIndex ) const {return mRendererScalarSettings.contains( groupIndex );}
+
+    /**
+     * Removes scalar settings with \a groupIndex
+     * \since QGIS 3.30.2
+     */
+    bool removeScalarSettings( int groupIndex )  {return mRendererScalarSettings.remove( groupIndex );}
+
     //! Returns renderer settings
     QgsMeshRendererVectorSettings vectorSettings( int groupIndex ) const { return mRendererVectorSettings.value( groupIndex ); }
     //! Sets new renderer settings
     void setVectorSettings( int groupIndex, const QgsMeshRendererVectorSettings &settings ) { mRendererVectorSettings[groupIndex] = settings; }
 
+    /**
+     * Returns whether \a groupIndex has existing vector settings
+     * \since QGIS 3.30.2
+     */
+    bool hasVectorSettings( int groupIndex ) const {return mRendererVectorSettings.contains( groupIndex );}
+
+    /**
+     * Removes vector settings for \a groupIndex
+     * \since QGIS 3.30.2
+     */
+    bool removeVectorSettings( int groupIndex )  {return mRendererVectorSettings.remove( groupIndex );}
+
     /**
      * Returns averaging method for conversion of 3d stacked mesh data to 2d data
      *
      * Caller does not own the resulting pointer
      */
-    QgsMesh3dAveragingMethod *averagingMethod() const;
+    QgsMesh3DAveragingMethod *averagingMethod() const;
 
     /**
      * Sets averaging method for conversion of 3d stacked mesh data to 2d data
      *
      * Ownership of the method is not transferred.
      */
-    void setAveragingMethod( QgsMesh3dAveragingMethod *method );
+    void setAveragingMethod( QgsMesh3DAveragingMethod *method );
 
     //! Writes configuration to a new DOM element
     QDomElement writeXml( QDomDocument &doc, const QgsReadWriteContext &context = QgsReadWriteContext() ) const;
@@ -673,6 +786,13 @@ class CORE_EXPORT QgsMeshRendererSettings
      */
     void setActiveVectorDatasetGroup( int activeVectorDatasetGroup );
 
+    /**
+    * Returns whether the group with \a index has render settings (scalar or vector)
+    *
+    * \since QGIS 3.22
+    */
+    bool hasSettings( int datasetGroupIndex ) const;
+
   private:
     QgsMeshRendererMeshSettings mRendererNativeMeshSettings;
     QgsMeshRendererMeshSettings mRendererTriangularMeshSettings;
@@ -688,7 +808,7 @@ class CORE_EXPORT QgsMeshRendererSettings
     int mActiveVectorDatasetGroup = -1;
 
     //! Averaging method to get 2D datasets from 3D stacked mesh datasets
-    std::shared_ptr<QgsMesh3dAveragingMethod> mAveragingMethod;
+    std::shared_ptr<QgsMesh3DAveragingMethod> mAveragingMethod;
 };
 
 #endif //QGSMESHRENDERERSETTINGS_H
